﻿using System.Reflection;
using PSFNetObjects;
using Shared.Models;
using System;
using System.Collections.ObjectModel;
using System.Data;
using System.Diagnostics;
using System.Globalization;
using System.Threading;

namespace Shared
{
    public sealed class Utilities
    {
        #region Debug Methods

        /// <summary>
        /// Outputs a message to the debug log.
        /// </summary>
        /// <param name="message">the message to output</param>
        public static void TraceMessage(string message)
        {
            if (string.IsNullOrEmpty(message))
                return;

            Trace.WriteLine(string.Format(CultureInfo.InvariantCulture, "[{0:HH:mm:ss}][{2}] - {1}", DateTime.Now, message, Assembly.GetExecutingAssembly().GetName().Name));
        }

        /// <summary>
        /// Traces an exception to the debug log.
        /// </summary>
        /// <param name="ex">The exception to output</param>
        public static void TraceException(Exception ex)
        {
            if (ex != null)
            {
                if (ex is ThreadAbortException)
                    return;

                Trace.WriteLine("-------An exception was caught--------");
                Trace.WriteLine(ex.Message);
                Trace.WriteLine("-------------Stack Trace--------------");
                Trace.WriteLine(ex.StackTrace);

                Exception iex = ex.InnerException;
                while (iex != null)
                {
                    Trace.WriteLine("-----------Inner Exception------------");
                    Trace.WriteLine(iex.Message);
                    Trace.WriteLine(iex.StackTrace);
                    iex = iex.InnerException;
                }
                Trace.WriteLine("--------------------------------------");
            }
            else
            {
                TraceMessage("An exception was raised....The exception itself was null.");
            }
        }

        #endregion

        #region Get Record Value
        /// <summary>
        /// Retrives a value from a data record
        /// </summary>
        /// <typeparam name="T">The datatype to return</typeparam>
        /// <param name="record">the record containing the value</param>
        /// <param name="columnName">the name of the column containing the value</param>
        /// <returns>the value of the specified column as the specified type is possible, if this fails the default value for the type is returned</returns>
        public static T GetRecordValue<T>(IDataRecord record, string columnName)
        {
            if (record == null)
                throw new ArgumentNullException("record");

            T result;
            try
            {
                result = record[columnName] is DBNull ? default(T) : (T)record[columnName];
            }
            catch (InvalidCastException)
            {
                return default(T);
            }
            catch (IndexOutOfRangeException)
            {
                return default(T);
            }

            return result;
        }

        /// <summary>
        /// Retrives a value from a data row
        /// </summary>
        /// <typeparam name="T">The datatype to return</typeparam>
        /// <param name="row">The data row containing  the value</param>
        /// <param name="columnName">The name of the column containing the value</param>
        /// <returns>the value of the specified column as the specified type is possible, if this fails the default value for the type is returned</returns>
        public static T GetRecordValue<T>(DataRow row, string columnName)
        {
            if (row == null)
                throw new ArgumentNullException("row");

            T result;
            try
            {
                result = row[columnName] is DBNull ? default(T) : (T)row[columnName];
            }
            catch (InvalidCastException)
            {
                return default(T);
            }
            catch (IndexOutOfRangeException)
            {
                return default(T);
            }
            catch (ArgumentException)
            {
                return default(T);
            }

            return result;
        }

        #endregion

        #region Username List

        /// <summary>
        /// Loads a list of Financials users from the specified server
        /// </summary>
        /// <param name="server">the name of the server</param>
        /// <returns>a list of PSF users</returns>
        public static Collection<User> UserNameList(string server)
        {
            if (string.IsNullOrEmpty(server))
                throw new ArgumentException("server cannot be empty", "server");

            Utilities.TraceMessage("Loading username list.");
            Collection<User> users = new Collection<User>();
            try
            {
                using (DataTable usernameInfoList = PSFNetSystem.UserList(server, true).Tables[0])
                {
                    foreach (DataRow row in usernameInfoList.Rows)
                        users.Add(User.FromDataRow(row));
                }
            }
            catch (Exception ex)
            {
                Utilities.TraceException(ex);
                throw;
            }

            Utilities.TraceMessage("Username list loaded successfully.");
            return users;
        }

        #endregion

        #region Converters

        //public static ObservableCollection<T> ConvertListToObservableCollection(T)
        //{

        //}

        #endregion
    }
}
